%This is an equivalent stiffness model of a fuel cell stack which calculates 
%component compression and clamping pressures based on simple geometry. 
%The model is detailed in the paper titled 'Equivalent stiffness model of a 
%PEM fuel cell stack including hygrothermal effects and dimensional
%tolerances' Published in the ASME journal of electrochemical energy
%conversion and storage (JEECS-17-1077).
%
%The user inputs cell geometry, components mechanical properties, number of
%cells in stack, and manufacturing tolerances. The model outputs the
%stack compression at different forces, clamping pressure across the
%MEA, compression percentage of the different components and the elongation
%required by the clamping method.
%
%The model considers temperature and hydration effects, including
%the change in PEM Young's modulus and swelling with temperature and 
%humidity.  Different MEA and gasket thickness can also be modelled through
%contact, both before and after operating conditions have been applied.
%Manufacturig thickness tolerances and temperature and humidity variation
%may also be applied to represent real world cell to cell variation.
%Non-linear compression of the gasket and GDL are considered.
%
% Dr. Ashley Fly, Loughborough University, 2017
% _______________________________________
%|                                       |  Endplate (not modelled)
%|_______________________________________|
% ||  _______________________________  ||
% || |   _   _   _   _   _   _   _   | ||   Bi-polar plate    
% || |__| |_| |_| |_| |_| |_| |_| |__| ||
% ||  _  _________________________  _  ||
% || |_||_________________________||_| ||   MEA (2*GDL+PEM) and gasket 
% ||  ____   _   _   _   _   _   ____  ||     
% || |    |_| |_| |_| |_| |_| |_|    | ||   Bi-polar plate
% || |_______________________________| ||
% ||___________________________________||
%|                                       |  Endplate (not modelled)
%|_______________________________________|


%clear all,%close all

run('GDL_data.m')

%%%%%%%%  MODEL OPTIONS %%%%%%%%
Variable_thickness = 0;   %Switch to 1 for variable membrane, GDL and gasket thickness, 0 for constant
Variable_hydration_sin = 0;
Variable_temperature = 0; %Swtich to 1 for variable cell temperature, 0 for constant
Variable_temprature_sin = 0;Variable_hydration_random = 0;   %Switch to 1 for variable cell hydration, 0 for constant
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N_cells =  100; %Number cells in stack
T_operation = 338; %Temperature at operation [K]
T_assembly = 298.15; %Temperature at stack assembly [K]
a_operation = 0.8; %Water activity (relative humidity) during operation
a_assembly = 0.3; %Water activity (relative humidity) during assembly


%%%%% Component properties %%%%%

% Bi-polar plate
w_bpp = 0.11; %Bi-polar plate width [m]
h_bpp = 0.11; %Bi-polar plate height [m]
w_bpp_ribs = 0.1; %Bi-polar plate ribbed area width [m]
h_bpp_ribs = 0.1; %Bi-polar plate ribbed area height [m]
w_rib = 1e-3; %Bi-polar plate rib width (single) [m]
n_ribs = 46; %Number ribs in bi-polar plate
t_bpp = 3e-3; %Bi-polar plate thickness [m]
d_rib = 1e-3; %Bi-polar plate rib depth [m]
E_bpp = 10e9; %Young's modulus bi-polar plate material [Pa] (Y.Zhou et al. Journal Power Sources 192 (2009) 544-551)
alpha_bpp = 7.9e-6; %Coefficient linear thermal expansion bi-polar plate [K^-1] (P.Lin et al. Journal Power Sources 194 (2009) 381-390)

%Gas diffusion layer
w_gdl = w_bpp_ribs; %Gas diffusion layer width [m]
h_gdl = h_bpp_ribs; %Gas diffusion layer height [m]
t_gdl = 200e-6; %Gas diffusion layer thickness [m]
E_gdl = 15e6; %Young's modulus gas diffusion layer material [Pa] (M.Ismail et al. Fuel Cells 12 (2012), 391-397), non-lineaer stiffness considered in script
alpha_gdl = 7.9e-6; %Coefficient linear thermal expansion gas diffusion layer [K^-1] (P.Lin et al. Journal Power Sources 194 (2009) 381-390)

%Polymer electrolyte membrane
w_pem = w_bpp_ribs; %Polymer electrolyte membrane width [m]
h_pem = h_bpp_ribs; %Polymer electrolyte membrane height [m]
t_pem = 50e-6; %Polymer electrolyte membrane thickness [m]
alpha_pem = 90e-6; %Coefficient linear thermal expansion polymer electrolyte membrane [K^-1] (P.Lin et al. Journal Power Sources 194 (2009) 381-390)
beta_pem = 0.0115; %Coefficient of linear expansion in polymer electrolyte membrane due to hydration [/lambda] (Y.Zhou et al. Journal Power Sources 192 (2009) 544-551)

%Gasket
w_gasket_outer = 0.108; %Gasket outer width [m]
h_gasket_outer = w_gasket_outer; %Gasket outer_height [m]
w_gasket_inner = w_bpp_ribs; %Gasket inner width [m]
h_gasket_inner = h_bpp_ribs; %Gasket inner height [m]
t_gasket_single = t_gdl; % Gasket thickness [m]
E_gasket = 50e6; %Young's modulus gasket material [Pa] (P.Lin et al. Journal Power Sources 194 (2009) 381-390), non-linear stifness considered in script
alpha_gasket = 77e-6; %Coefficient linear thermal expansion gasket [K^-1] (P.Lin et al. Journal Power Sources 194 (2009) 381-390)

%Clamp (fastening device)
r_clamp = 4e-3; %Clamp radius [m]
n_clamp = 8; %Number clamps on cell
E_clamp = 180e9; %Young's modulus clamping material [Pa]
alpha_clamp = 12e-6; %Coefficient linear thermal expansion clamp [K^-1]


%MEA
t_mea = t_pem + t_gdl*2; %MEA thickness [m]
Lambda_operation = 0.0043+17.81*a_operation-39.85*a_operation^2+36*a_operation^3; %Membrane water content H_2O/SO3- sites (Springer er al. ECS (1991))
Lambda_assembly = 0.0043+17.81*a_assembly-39.85*a_assembly^2+36*a_assembly^3; %Membrane water content H_2O/SO3- sites (Springer er al. ECS (1991))

%Matrix of PEM Young's modulus values as function of temperature and
%humidity from Y.Zhou et al./ Journal Power Sources 192 (2009) 544-551
E_pem_matrix = [197 192 132 121; 161 137 103 70; 148 117 92 63; 121 89 59 46]*1e6; %PEM Young's Modulus [N/m^2]
T_pem_vector = [25 45 65 85]+273.15; %Temperature of PEM (for lookup matrix) [K]
RH_pem_vector = [30 50 70 90]/100; %Releative humidity of PEM (for lookup matrix)
%Conversion of humidity into water content (\lambda) using correlation of Springer et.al, 1991, ECS
Lambda_pem_vector = 0.0043+17.81*RH_pem_vector-39.85*RH_pem_vector.^2+36*RH_pem_vector.^3; 

%Different options for Young's modulus
%%(Constant)
%E_pem_assembly = 190e6; %Young's modulus Polymer electrolyte membrane material CONSTANT [Pa]
%E_pem_operation = 190e6; %Young's modulus Polymer electrolyte membrane material CONSTANT [Pa]
%%(Direct 2D lookup)
%E_pem_assembly = interp2(Lambda_pem_vector,T_pem_vector,E_pem_matrix,Lambda_assembly,T_assembly); %Young's modulus Polymer electrolyte membrane material VARIABLE [Pa]
%E_pem_operation = interp2(Lambda_pem_vector,T_pem_vector,E_pem_matrix,Lambda_operation,T_operation); %Young's modulus Polymer electrolyte membrane material VARIABLE [Pa]
%%(2nd order surface fit), gives 0.971 R^2 value of fit
E_pem_assembly = 1e6*(363.8-46.08*Lambda_assembly-2.6*(T_assembly-273.15)+2.61*Lambda_assembly^2+0.033*Lambda_assembly*(T_assembly-273.15)+0.01*(T_assembly-273.15)^2); %Young's modulus Polymer electrolyte membrane material VARIABLE [Pa]
%E_pem_operation is calculated inside the loop stage since this can vary
%cell to cell with hydration and temperature

%Warn user if selected humidity or temperature is out of correlation range
if Lambda_operation < min(Lambda_pem_vector) || Lambda_operation > max(Lambda_pem_vector) || Lambda_assembly < min(Lambda_pem_vector) || Lambda_assembly > max(Lambda_pem_vector) 
    disp('ERROR: Hydration value out of range!')
    %return
elseif T_operation < min(T_pem_vector) || T_operation > max(T_pem_vector) || T_assembly < min(T_pem_vector) || T_assembly > max(T_pem_vector)
    disp('ERROR: Temperature value out of range!')
    %return
end
    
%%%%%% Cross sectional area calculations %%%%%%
A_bpp_main = h_bpp*w_bpp; %Cross sectional area bi-polar plate main section [m^2]
A_bpp_rib_edges = h_bpp*w_bpp-h_bpp_ribs*w_bpp_ribs; %Cross sectional area bi-polar plate outside edges [m^2]
A_bpp_ribs = w_rib*h_bpp_ribs*n_ribs; %Cross sectional area bi-polar plate ribs [m^2]
L_bpp_main = t_bpp-d_rib; %Thickness bi-polar plate main section [m]
A_gdl =0.5355*w_gdl*h_gdl; %Cross sectional area gas diffusion layer [m^2] (Factor of 0.5355 represents contact area between ribs and GDL)
A_pem_cell =0.5355*w_pem*h_pem; %Cross sectional area polymer electrolyte membrane under GDL [m^2] (Factor of 0.5355 represents contact area between ribs and GDL)
A_gasket = w_gasket_outer*h_gasket_outer-w_gasket_inner*h_gasket_inner; %Cross sectional area gasket [m^2]
A_pem_gasket = A_gasket; %Cross sectional area of polymer electrolyte membrane under gasket [m^2]
A_clamp = pi*r_clamp^2*n_clamp; %Clamp area [m^2]

%%%%% Stiffness calculations (k=EA/L) %%%%%
k_bpp_main = E_bpp*A_bpp_main/L_bpp_main; %Stiffness of bpp main section [N/m]
k_bpp_rib_edges = E_bpp*A_bpp_rib_edges/d_rib; %Stiffness of bpp rib edges [N/m]
k_bpp_ribs = E_bpp*A_bpp_ribs/d_rib; %Stiffness of bpp ribs [N/m]
k_bpp_constant = 1/(1/(k_bpp_ribs+k_bpp_rib_edges)+1/k_bpp_main); %Stiffness of bi-polar plate combined [N/m]

if Variable_thickness %If variable thickness selected assign random thickness variation to each component    
    sd_gdl = t_gdl*0.005; %Standard deviation of GDL (for variable thickness) [m]
    sd_pem = t_pem*0.005; %Standard deviation of PEM (for variable thickness) [m]
    sd_gasket = t_gasket_single*0.005; %Standard deviation of gasket (for variable thickness) [m]
    t_gdl_array_separate = [normrnd(t_gdl,sd_gdl,1,N_cells); normrnd(t_gdl,sd_gdl,1,N_cells)]; %Array of GDL thickness, one row anode GDL, one row cathode GDL [m]
    t_pem_array = normrnd(t_pem,sd_pem,1,N_cells); %Array of PEM thickness with random variation [m]
    t_gasket_array_separate = [normrnd(t_gasket_single,sd_gasket,1,N_cells); normrnd(t_gasket_single,sd_gasket,1,N_cells)]; %Array of gasket thickness with random variation, one row each side of PEM [m]
else %Equal thickness in each cell, populate array with equal numbers
    t_gdl_array_separate = [t_gdl*ones(1,N_cells); t_gdl*ones(1,N_cells)]; %Array of GDL thickness, no variation [m]
    t_pem_array = t_pem*ones(1,N_cells); %Array of PEM thickness, no variation [m]
    t_gasket_array_separate = [t_gasket_single*ones(1,N_cells); t_gasket_single*ones(1,N_cells)]; %Array of gasket thickness, no variation [m]
end
t_gasket_array = sum(t_gasket_array_separate); %Total gasket thickness per cell (each side of PEM) [m]
t_gasket_ass_array = t_gasket_array+t_pem_array; %Total gasket assembly thickness (gasket-PEM-gasket) [m]
t_gdl_array = sum(t_gdl_array_separate); %Total GDL thickness per cell (anode plus cathode) [m]
t_mea_array = t_gdl_array+t_pem_array; %Total MEA thickness per cell [m]
delta = t_mea_array-t_gasket_ass_array; %Difference between GDL and gasket thickness for each cell in stack [m]

if Variable_hydration_random %If variable hydration assign random hydration variation to each cell
    sd_lambda = Lambda_operation*0.05; %Standard deviation of membrane water content
    Lambda_operation_array = normrnd(Lambda_operation,sd_lambda,1,N_cells); %Array of membrane water content per cell
elseif Variable_hydration_sin %If variable hydration assign a structured hydration variation to each cells (i.e. dry cells at edge of stack)
    Lambda_operation_array = ones(1,N_cells)*Lambda_operation+Lambda_change*sin((1:(N_cells))*pi/(N_cells+1))-Lambda_change*sin(pi/(N_cells+1)); %More humid at centre of stack 
else %Equal hydration in each cell, populate array with equal numbers
    Lambda_operation_array = Lambda_operation*ones(1,N_cells); %Array of membrane water content per cell
end

if Variable_temperature %If variable temperature assign random variation to each cell
    sd_temp = T_operation*0.01; %Standard deviation of cell temperature [C]
    T_operation_array = normrnd(T_operation,sd_temp,1,N_cells); %Array of cell temperatures [K]
elseif Variable_temprature_sin %If variable hydration assign a structured hydration variation to each cells (i.e. cooler cells at end of stack)
    T_operation_array = ones(1,N_cells)*T_operation+T_change*sin((1:(N_cells))*pi/(N_cells+1))-T_change*sin(pi/(N_cells+1)); %Warmer at centre of stack 
else %Equal hydration in each cell, populate array with equal numbers
    T_operation_array = T_operation*ones(1,N_cells); %Array of cell temperatures [K] 
end

%%%%% Calculations of component thermal and hydration expansion (delta_L = alpha*delta_T*L) %%%%%
delta_L_bpp = alpha_bpp*(T_operation_array-T_assembly)*t_bpp; %Thermal expansion of a single bpp (separate value for each cell) [m]
delta_L_gdl = alpha_gdl*(T_operation_array-T_assembly).*t_gdl_array; %Thermal expansion of a single gdl (separate value for each cell)[m]
delta_L_pem_thermal = alpha_pem*(T_operation_array-T_assembly).*t_pem_array; %Thermal expansion of a single pem (separate value for each cell) [m]
delta_L_pem_hydration = beta_pem*(Lambda_operation_array-Lambda_assembly).*t_pem_array; %Hydraulic expansion of a single PEM (separate value for each cell)[m]
delta_L_pem = delta_L_pem_thermal+delta_L_pem_hydration; %Total expansion of a single PEM (separate value for each cell)[m]
delta_L_gasket = alpha_gasket*(T_operation_array-T_assembly).*t_gasket_array; %Thermal expansion of a single gasket (separate value for each cell)[m]
delta_L_gasket_assembly = delta_L_gasket+delta_L_pem; %Thermal expansion of single gasket assembly [m] 
delta_L_mea = delta_L_gdl+delta_L_pem; %Thermal expansion of single mea (separate value for each cell) [m]

t_bpp_thermal_array = t_bpp+delta_L_bpp; %Thickness of the BPP including thermal expansion [m]
t_gdl_thermal_array = t_gdl_array+delta_L_gdl; %Thickness of the GDL including thermal expansion [m]
t_pem_thermal_array = t_pem_array+delta_L_pem; %Thickness of the PEM including thermal expansion [m]
t_gasket_thermal_array = t_gasket_array+delta_L_gasket; %Thickness of gaskets after thermal expansion [m]
t_gasket_ass_thermal_array = t_gasket_ass_array+delta_L_gasket_assembly; %Thickness of the gasket including thermal expansion [m]
t_mea_thermal_array = t_mea_array+delta_L_mea; %Thickness of the MEA including thermal expansion [m]
delta_thermal = t_mea_thermal_array-t_gasket_ass_thermal_array; %Difference between mea and gasket thickness after temperature change (single cell) [m]

t_stack = t_bpp*(N_cells+1)+sum(max([t_mea_array; t_gasket_ass_array])); %Stack thickness (uncompressed) [m]
t_stack_thermal = sum(t_bpp_thermal_array)+mean(t_bpp_thermal_array)+sum(max([t_mea_thermal_array; t_gasket_ass_thermal_array])); %Stack thickness at temperature (uncompressed) [m]
delta_L_stack = t_stack_thermal-t_stack-1e-9; %Expansion of stack due to thermal and hydration effects [m] (Note: model cannot calculate for stack shrinkage)

%%%% Determine component stiffness arrays (k=EA/t) %%%%
%Assembly condition
k_pem_constant_cell_assembly = E_pem_assembly*A_pem_cell./t_pem_array; %Constant spring stiffness of pem for cell m [N/m]
k_pem_constant_gasket_assembly = E_pem_assembly*A_pem_gasket./t_pem_array; %Constant spring stiffness of pem for cell m [N/m]
k_gdl_constant = E_gdl*A_gdl./t_gdl_array; %Constant stiffness of gdl for cell m [N/m]
k_gdl_single = E_gdl*A_gdl./mean(t_gdl_array_separate); %Constant stiffness of gdl for cell m [N/m]
k_gasket_constant = E_gasket*A_gasket./t_gasket_array; %Constant stiffness gasket [N/m]
k_gasket_single = E_gasket*A_gasket./mean(t_gasket_array_separate); %Constant stiffness gasket [N/m]
k_mea_constant = 1./(1./k_gdl_constant+1./k_pem_constant_cell_assembly); %Constant stiffness of mea for cell m [N/m]
%Operation condition
E_pem_operation = 1e6*(363.8-46.08*Lambda_operation_array-2.6*(T_operation_array-273.15)+2.61*Lambda_operation_array.^2+0.033*Lambda_operation_array.*(T_operation_array-273.15)+0.01*(T_operation_array-273.15).^2);
k_pem_constant_cell_thermal = E_pem_operation*A_pem_cell./t_pem_thermal_array; %Constant spring stiffness of pem [N/m]
k_pem_constant_gasket_thermal = E_pem_operation*A_pem_gasket./t_pem_thermal_array; %Constant spring stiffness of pem [N/m]
k_gdl_constant_thermal = E_gdl*A_gdl./t_gdl_thermal_array; %Constant stiffness of gdl for cell m [N/m]
k_gdl_single_thermal = E_gdl*A_gdl./(mean(t_gdl_array_separate)+0.5*delta_L_gdl); %Constant stiffness of gdl for cell m [N/m]
k_gasket_single_thermal = E_gasket*A_gasket./(mean(t_gasket_array_separate)+0.5*delta_L_gasket); %Constant stiffness gasket [N/m]
k_mea_constant_thermal = 1./(1./k_gdl_constant_thermal+1./k_pem_constant_cell_thermal); %Constant stiffness of mea for cell m [N/m]
k_gasket_constant_thermal = E_gasket*A_gasket./t_gasket_thermal_array; %Constant stiffness gasket [N/m]
k_gasket_constant_ass_thermal = 1./(1./k_gasket_constant_thermal+1./k_pem_constant_gasket_thermal); %Constant stiffness of mea for cell m [N/m]

%Initialise non-linear gasket and GDL behaviour
x_gdl = linspace(1e-10,t_gdl*0.8,1e4); %GDL displacemtn input vector [m]
k_gdl_scaling_factor=smooth(interp1(SGL10BA_m(:,1)*t_gdl/t_SGL10BA,SGL10BA_m(:,3),x_gdl,'linear',max(SGL10BA_m(:,3)))/max(SGL10BA_m(:,3)),100)'; %Stiffness scaling factor based on Young's modulus empricial value (1=max stiffness), non-linear curve from GDL_data.m
x_gasket = linspace(1e-10,t_gasket_single*0.8,1e4); %Gasket displacemtn input vector [m]
k_gasket_scaling_factor=1.7*smooth(interp1(Samco_Gasket_m(:,1)*t_gasket_single/t_Samco_Gasket,Samco_Gasket_m(:,3),x_gasket,'linear',max(Samco_Gasket_m(:,3)))/max(Samco_Gasket_m(:,3)),100)'; %Stiffness scaling factor based on Young's modulus empricial value (1=max stiffness), non-linear curve from GDL_data.m
%k_gasket_scaling_factor=1.7*ones(1,length(x_gasket)); %Use for constant stiffness (linear model), scaling factor equals 1.7
%k_gdl_scaling_factor=ones(1,length(x_gdl)); %Use for constant stiffness


%%%% Initialise variables for loop %%%%
x = linspace(1e-10,1e-4*N_cells,1e4); %Displacement vector [m] (Distance over which force for compression will be found)
sympref('HeavisideAtOrigin', 0); %Changes Heaviside function value at crossover to 0 (used for touch vector)
touch_vector_bpp = ones(1,length(x)); %Vector to indicate that bpp is in contact at all times (1=touch, 0=gap)
k_bpp = k_bpp_constant*touch_vector_bpp; %Stiffness of bi-polar plate at different displacements [N/m]

%Loop1 - Initialise empty vectors for efficiency
touch_vector_gasket = zeros(N_cells,length(x)); touch_vector_mea = zeros(N_cells,length(x)); k_pem_cell = zeros(N_cells,length(x)); k_gdl = zeros(N_cells,length(x)); k_mea = zeros(N_cells,length(x)); k_gasket_ass = zeros(N_cells,length(x)); k_gasket = zeros(N_cells,length(x));k_gasket_thermal = zeros(N_cells,length(x));
touch_vector_gasket_thermal = zeros(N_cells,length(x));touch_vector_mea_thermal = zeros(N_cells,length(x));k_pem_cell_thermal = zeros(N_cells,length(x));k_gdl_thermal = zeros(N_cells,length(x));k_mea_thermal = zeros(N_cells,length(x));k_gasket_ass_thermal = zeros(N_cells,length(x));
F_required_mea = zeros(N_cells,length(x));F_required_gasket = zeros(N_cells,length(x));F_required_cell = zeros(N_cells,length(x));
F_required_mea_thermal = zeros(N_cells,length(x));F_required_gasket_thermal = zeros(N_cells,length(x));F_required_cell_thermal = zeros(N_cells,length(x));
%Loop2 - Initialise empty vectors for efficiency
x2 = ones(1,1e4+1); %Vector to set variable length for loop 2 (Different length to loop 1 to avoid cross use of variables)
compression_cell_area_single = zeros(N_cells,length(x2));F_mea = zeros(N_cells,length(x2));F_gasket = zeros(N_cells,length(x2));
compression_mea_single = zeros(N_cells,length(x2));compression_pem_single = zeros(N_cells,length(x2));compression_gdl_single = zeros(N_cells,length(x2));compression_gasket_single = zeros(N_cells,length(x2));
compression_cell_area_single_thermal = zeros(N_cells,length(x2));F_mea_thermal = zeros(N_cells,length(x2));F_gasket_thermal = zeros(N_cells,length(x2));compression_mea_single_thermal = zeros(N_cells,length(x2));compression_pem_single_thermal = zeros(N_cells,length(x2));compression_gdl_single_thermal = zeros(N_cells,length(x2));compression_gasket_single_thermal = zeros(N_cells,length(x2));
compression_pem = zeros(N_cells,length(x2)); compression_gdl = zeros(N_cells,length(x2)); compression_gasket = zeros(N_cells,length(x2));k_mea_F = zeros(N_cells,length(x2));k_gdl_F = zeros(N_cells,length(x2));k_gasket_F = zeros(N_cells,length(x2));k_mea_thermal_F = zeros(N_cells,length(x2));k_gdl_thermal_F = zeros(N_cells,length(x2));k_gasket_thermal_F = zeros(N_cells,length(x2));
%Loop3 - Initialise empty vectors for efficiency
P_mea_change_fixed = zeros(N_cells,length(x2)); P_gasket_change_fixed = zeros(N_cells,length(x2)); compression_pem_free_thermal = zeros(N_cells,length(x2)); compression_gdl_free_thermal = zeros(N_cells,length(x2)); compression_gasket_free_thermal = zeros(N_cells,length(x2));

for m=1:N_cells %Begin loop to calculate force compression relationship for individial stack components

    %%%Non-hygrothermal (asembly condition)%%%
    t_mea = t_mea_array(m); %Thickness of MEA for cell m
    touch_vector_gasket(m,:) = heaviside(x-delta(m)); %Vector to indicate if gasket is touching bpp assuming even compression between cells(1=touch, 0=gap)
    touch_vector_mea(m,:) = heaviside(x+delta(m)); %Vector to indicate if mea is touching bpp assuming even compression between cells  (1=touch, 0=gap)
   
    %Determine non-linear stiffness of gas diffusion layer% NOTE: does not consider thickness difference between two GDLs in one cell
    k_gdl_nl = k_gdl_single(m).*k_gdl_scaling_factor; %GDL stiffness as a function of GDL displacement [N/m]
    F_mea_comp = ((1./((2./k_gdl_nl)+(1/k_pem_constant_cell_assembly(m)))).*2.*x_gdl)./(1-(1./((2*k_pem_constant_cell_assembly(m)./k_gdl_nl)+1))); %Force required to compress MEA by GDL deflection x_gdl [N]
    x_mea = 2*x_gdl+F_mea_comp/k_pem_constant_cell_assembly(m); %MEA deflection corresponding to force and GDL deflection [N] (NOT CONSIDERING SPACING)
    k_mea_no_spacing = F_mea_comp./x_mea; %MEA stiffness [N/m] (NOT CONSIDERING SPACING)
    k_mea(m,:) = interp1(x_mea,k_mea_no_spacing,max(1e-9,min(x,x+delta(m))),'linear',max(k_mea_no_spacing)).*touch_vector_mea(m,:); %MEA stiffness in terms of displacement x [N/m]
    k_gdl(m,:) = (2*k_pem_constant_cell_assembly(m).*k_mea(m,:))./(k_pem_constant_cell_assembly(m)-k_mea(m,:));
    %Determine non-linear stiffness of gasket% NOTE: does not consider thickness difference between two gaskets in one cell
    k_gasket_nl = k_gasket_single(m).*k_gasket_scaling_factor; %Gasket stiffness as a function of gasket displacement [N/m]
    F_gasket_comp = ((1./((2./k_gasket_nl)+(1/k_pem_constant_gasket_assembly(m)))).*2.*x_gasket)./(1-(1./((2*k_pem_constant_gasket_assembly(m)./k_gasket_nl)+1))); %Force required to compress Gasket assembly by gasket deflection x_gasket [N]
    x_gasket_assembly = 2*x_gasket+F_gasket_comp/k_pem_constant_gasket_assembly(m); %Gasket assembly deflection corresponding to force and gasket deflection deflection [N] (NOT CONSIDERING SPACING)
    k_gasket_no_spacing = F_gasket_comp./x_gasket_assembly; %Gasket assembly stiffness [N/m] (NOT CONSIDERING SPACING)
    k_gasket_ass(m,:) = interp1(x_gasket_assembly,k_gasket_no_spacing,max(1e-9,min(x,x-delta(m))),'linear',max(k_gasket_no_spacing)).*touch_vector_gasket(m,:); %Gasket assembly stiffness in terms of displacement x [N/m]
    k_gasket(m,:) = (2*k_pem_constant_gasket_assembly(m).*k_gasket_ass(m,:))./(k_pem_constant_gasket_assembly(m)-k_gasket_ass(m,:));
    
    %%%Thermal%%%
    %Stiffness vectors are recalculated after temperature change as different thermal expansion coefficients could change components contact status
    touch_vector_gasket_thermal(m,:) = heaviside(x-delta_thermal(m)); %Vector to indicate if gasket is touching bpp assuming even compression between cells after temperature change(1=touch, 0=gap)
    touch_vector_mea_thermal(m,:) = heaviside(x+delta_thermal(m)); %Vector to indicate if mea is touching bpp assuming even compression between cells  (1=touch, 0=gap)
    k_pem_cell_thermal(m,:) = k_pem_constant_cell_thermal(m)*touch_vector_mea_thermal(m,:); %Stiffness of PEM at different displacements after temperature change (k=0 if no contact) [N/m]
    
    k_gdl_nl_thermal = k_gdl_single_thermal(m).*k_gdl_scaling_factor; %GDL stiffness as a function of GDL displacement [N/m]
    F_mea_comp_thermal = ((1./((2./k_gdl_nl_thermal)+(1/k_pem_constant_cell_thermal(m)))).*2.*x_gdl)./(1-(1./((2*k_pem_constant_cell_thermal(m)./k_gdl_nl_thermal)+1))); %Force required to compress MEA by GDL deflection x_gdl [N]
    x_mea_thermal = 2*x_gdl+F_mea_comp_thermal/k_pem_constant_cell_thermal(m); %MEA deflection corresponding to force and GDL deflection [N] (NOT CONSIDERING SPACING)
    k_mea_no_spacing_thermal = F_mea_comp_thermal./x_mea_thermal; %MEA stiffness [N/m] (NOT CONSIDERING SPACING)
    k_mea_thermal(m,:) = interp1(x_mea_thermal,k_mea_no_spacing_thermal,max(1e-9,min(x,x+delta_thermal(m))),'linear',max(k_mea_no_spacing_thermal)).*touch_vector_mea_thermal(m,:); %MEA stiffness in terms of displacement x [N/m]
    k_gdl_thermal(m,:) = (2*k_pem_constant_cell_thermal(m).*k_mea_thermal(m,:))./(k_pem_constant_cell_thermal(m)-k_mea_thermal(m,:));
    
    %Determine non-linear stiffness of gasket under thermal% NOTE: does not consider thickness difference between two gaskets in one cell
    k_gasket_nl_thermal = k_gasket_single_thermal(m).*k_gasket_scaling_factor; %Gasket stiffness as a function of gasket displacement [N/m]
    F_gasket_comp_thermal = ((1./((2./k_gasket_nl_thermal)+(1/k_pem_constant_gasket_thermal(m)))).*2.*x_gasket)./(1-(1./((2*k_pem_constant_gasket_thermal(m)./k_gasket_nl_thermal)+1))); %Force required to compress Gasket assembly by gasket deflection x_gasket [N]
    x_gasket_assembly_thermal = 2*x_gasket+F_gasket_comp_thermal/k_pem_constant_gasket_thermal(m); %Gasket assembly deflection corresponding to force and gasket deflection deflection [N] (NOT CONSIDERING SPACING)
    k_gasket_no_spacing_thermal = F_gasket_comp_thermal./x_gasket_assembly_thermal; %Gasket assembly stiffness [N/m] (NOT CONSIDERING SPACING)
    k_gasket_ass_thermal(m,:) = interp1(x_gasket_assembly_thermal,k_gasket_no_spacing_thermal,max(1e-9,min(x,x-delta_thermal(m))),'linear',max(k_gasket_no_spacing_thermal)).*touch_vector_gasket_thermal(m,:); %Gasket assembly stiffness in terms of displacement x [N/m]
    k_gasket_thermal(m,:) = (2*k_pem_constant_gasket_thermal(m).*k_gasket_ass_thermal(m,:))./(k_pem_constant_gasket_thermal(m)-k_gasket_ass_thermal(m,:));
    
    %%%%% Clamping calculations (F=kx) %%%%%
    F_required_bpp = k_bpp.*x; %Force required to compress a single bpp by distance x [N]
    %Non-thermal%
    if delta(m) < 0 %Gasket touch first
        F_required_mea(m,:) = k_mea(m,:).*(x+delta(m)); %Force required to compress MEA in single cell by distance x (F=0 for no contact since k=0) [N]
        F_required_gasket(m,:) = k_gasket_ass(m,:).*x; %Force required to compress gasket in single cell by distance x [N]
    elseif delta(m) > 0 %MEA touch first
        F_required_mea(m,:) = k_mea(m,:).*x; %Force required to compress MEA in single cell by distance x  [N]
        F_required_gasket(m,:) = k_gasket_ass(m,:).*(x-delta(m)); %Force required to compress gasket in single cell by distance x (F=0 for no contact since k=0) [N]
    else %Gasket and MEA touch simultaneously
        F_required_mea(m,:) = k_mea(m,:).*x; %Force required to compress MEA in single cell by distance x  [N]
        F_required_gasket(m,:) = k_gasket_ass(m,:).*x; %Force required to compress gasket in single cell by distance x [N]
    end
    F_required_cell(m,:) = F_required_mea(m,:)+F_required_gasket(m,:); %Force required to compress a single cell by distance x [N] 

    %Thermal%
    if delta_thermal(m) < 0 %Gasket touch first (under thermal conditions)
        F_required_mea_thermal(m,:) = k_mea_thermal(m,:).*(x+delta_thermal(m)); %Force required to compress MEA in single cell by distance x under thermal conditions (F=0 for no contact since k=0) [N]
        F_required_gasket_thermal(m,:) = k_gasket_ass_thermal(m,:).*x; %Force required to compress gasket in single cell by distance x under thermal conditions[N]
    elseif delta_thermal(m) > 0 %MEA touch first
        F_required_mea_thermal(m,:) = k_mea_thermal(m,:).*x; %Force required to compress MEA in single cell by distance x under thermal conditions  [N]
        F_required_gasket_thermal(m,:) = k_gasket_ass_thermal(m,:).*(x-delta_thermal(m)); %Force required to compress gasket in single cell by distance x under thermal conditions (F=0 for no contact since k=0) [N]
    else %Gasket and MEA touch simultaneously
        F_required_mea_thermal(m,:) = k_mea_thermal(m,:).*x; %Force required to compress MEA in single cell by distance x under thermal conditions  [N]
        F_required_gasket_thermal(m,:) = k_gasket_ass_thermal(m,:).*x; %Force required to compress gasket in single cell by distance x under thermal conditions [N]
    end
    F_required_cell_thermal(m,:) = F_required_mea_thermal(m,:)+F_required_gasket_thermal(m,:); %Force required to compress a single cell by distance x under thermal conditions [N] 

    
end
%The above loop calculates component compressive force required for a displacement x separately for each cell in the stack.
%The loop below uses this infomation to determine total stack compression for a specified force range
P_max = 10e6; %Maximum clamping pressure for study [Pa]

%Assembly condition%
fmin_cell = max([min(F_required_bpp),min(min(F_required_cell))]); %Highest minimum force for compression in cell [N]
fmax_cell = min([max(F_required_bpp),max(max(F_required_cell)),P_max*A_bpp_main]); %Lowest maximum forcce for compression [N]
F_range_cell = linspace(fmin_cell,fmax_cell,length(x2)); %Force range over which displacement to be found [N]
compression_bpp_single = interp1(F_required_bpp,x,F_range_cell); %Compression of single bpp for force range F [m]

%Operation condition%
fmin_cell_thermal = max([min(F_required_bpp),min(min(F_required_cell_thermal))]); %Highest minimum force for compression in cell [N]
fmax_cell_thermal = min([max(F_required_bpp),max(max(F_required_cell_thermal)),P_max*A_bpp_main]); %Lowest maximum forcce for compression [N]
F_range_cell_thermal = linspace(fmin_cell_thermal,fmax_cell_thermal,length(x2)); %Force range over which displacement to be found [N]
 compression_bpp_single_thermal = interp1(F_required_bpp,x,F_range_cell_thermal); %Compression of single bpp for force range F [m]

for m=1:N_cells 
    %Assembly condition%
    compression_cell_area_single(m,:) = interp1(F_required_cell(m,:),x,F_range_cell); %Compression of single cell area (MEA and gasket) for force range F [m]
    F_mea(m,:) = interp1(F_required_cell(m,:),F_required_cell(m,:)-F_required_gasket(m,:),F_range_cell); %Force through the MEA [N]
    F_gasket(m,:) = interp1(F_required_cell(m,:),F_required_cell(m,:)-F_required_mea(m,:),F_range_cell); %Force through the gasket [N]
    
    k_mea_F(m,:) = interp1(F_required_mea(m,max(1,find(F_required_mea(m,:),1,'first')-1):end),k_mea(m,max(1,find(F_required_mea(m,:),1,'first')-1):end),F_mea(m,:)); %Expression of MEA stiffness in commpression force vector [N/m]
    k_gdl_F(m,:) = interp1(F_required_mea(m,max(1,find(F_required_mea(m,:),1,'first')-1):end),k_gdl(m,max(1,find(F_required_mea(m,:),1,'first')-1):end),F_mea(m,:)); %Expression of GDL stiffness in compresison force vector [N/m]
    k_gasket_F(m,:) = interp1(F_required_gasket(m,max(1,find(F_required_gasket(m,:),1,'first')-1):end),k_gasket(m,max(1,find(F_required_gasket(m,:),1,'first')-1):end),F_gasket(m,:)); %Expression of gasket stiffness in commpression force vector [N/m]
    
    compression_mea_single(m,:) = F_mea(m,:)./k_mea_F(m,:); %Compression in single MEA [m]
    compression_pem_single(m,:) = F_mea(m,:)/k_pem_constant_cell_assembly(m); %Compression in single PEM [m]
    compression_gdl_single(m,:) = F_mea(m,:)./k_gdl_F(m,:); %Compression in single GDL [m]
    compression_gasket_single(m,:) = F_gasket(m,:)./k_gasket_F(m,:); %Compression in single gasket [m] 
    compression_pem(m,:) = (compression_pem_single(m,:))/t_pem_array(m)*100; %Percentage compression of polymer electrolyte membrane
    compression_gdl(m,:) = (compression_gdl_single(m,:))/mean(t_gdl_array_separate(:,m))*100; %Percentage compression of gas diffusion layer
    compression_gasket(m,:) = (compression_gasket_single(m,:))/mean(t_gasket_array_separate(:,m))*100; %Percentage compression of gasket

    %Thermal condition%
    compression_cell_area_single_thermal(m,:) = interp1(F_required_cell_thermal(m,:),x,F_range_cell_thermal); %Compression of single cell area (MEA and gasket) for force range F [m]
    F_mea_thermal(m,:) = interp1(F_required_cell_thermal(m,:),F_required_cell_thermal(m,:)-F_required_gasket_thermal(m,:),F_range_cell_thermal); %Force through the MEA [N]
    F_gasket_thermal(m,:) = interp1(F_required_cell_thermal(m,:),F_required_cell_thermal(m,:)-F_required_mea_thermal(m,:),F_range_cell_thermal); %Force through the gasket [N]
    
    k_mea_thermal_F(m,:) = interp1(F_required_mea_thermal(m,max(1,find(F_required_mea_thermal(m,:),1,'first')-1):end),k_mea_thermal(m,max(1,find(F_required_mea_thermal(m,:),1,'first')-1):end),F_mea_thermal(m,:)); %Expression of MEA stiffness in commpression force vector [N/m]
    k_gdl_thermal_F(m,:) = interp1(F_required_mea_thermal(m,max(1,find(F_required_mea_thermal(m,:),1,'first')-1):end),k_gdl_thermal(m,max(1,find(F_required_mea_thermal(m,:),1,'first')-1):end),F_mea_thermal(m,:)); %Expression of GDL stiffness in compresison force vector [N/m]
    k_gasket_thermal_F(m,:) = interp1(F_required_gasket_thermal(m,max(1,find(F_required_gasket_thermal(m,:),1,'first')-1):end),k_gasket_thermal(m,max(1,find(F_required_gasket_thermal(m,:),1,'first')-1):end),F_gasket(m,:)); %Expression of gasket stiffness in commpression force vector [N/m]
   
    compression_mea_single_thermal(m,:) = F_mea_thermal(m,:)./k_mea_thermal_F(m,:); %Compression in single MEA [m]
    compression_pem_single_thermal(m,:) = F_mea_thermal(m,:)/k_pem_constant_cell_thermal(m); %Compression in single PEM [m]
    compression_gdl_single_thermal(m,:) = F_mea_thermal(m,:)./k_gdl_thermal_F(m,:); %Compression in single GDL [m]
    compression_gasket_single_thermal(m,:) = F_gasket_thermal(m,:)./k_gasket_thermal_F(m,:); %Compression in single gasket [m] 
end

%Assembly condition
compression_bpp_total = compression_bpp_single*(N_cells+1); %Total compression of all BPPs in stack [m]
compression_cell_area_total = sum(compression_cell_area_single); %Total compression of all cell areas in stack [m]
compression_stack_total = compression_bpp_total+compression_cell_area_total; %Total compression of fuel cell stack for force range F [m]
t_stack_compressed = t_stack-compression_stack_total; %Stack thickness compressed[m]
%Operation condition
compression_bpp_total_thermal = compression_bpp_single_thermal*(N_cells+1); %Total compression of all bpps in stack [m]
compression_cell_area_total_thermal = sum(compression_cell_area_single_thermal); %Total compression of all cell areas in stack [m]
compression_stack_total_thermal = compression_bpp_total_thermal+compression_cell_area_total_thermal; %Total compression of fuel cell stack for force range F [m]
t_stack_compressed_thermal = t_stack_thermal-compression_stack_total_thermal; %Stack thickness compressed[m]

P_clamp = F_range_cell/A_bpp_main; %Cell clamping pressure [N/m^2]
P_mea = F_mea/A_pem_cell; %Contact pressure on MEA [N/m^2]
P_gasket = F_gasket/A_gasket; %Contact pressure on gasket [N/m^2]
P_clamp_thermal = F_range_cell_thermal/A_bpp_main; %Cell clamping pressure [N/m^2]
P_mea_thermal = F_mea_thermal/A_pem_cell; %Contact pressure on MEA [N/m^2]
P_gasket_thermal = F_gasket_thermal/A_gasket; %Contact pressure on gasket [N/m^2]
compression_stack = (t_stack-t_stack_compressed)/t_stack*100; %Percentage compression of stack
compression_bpp = (compression_bpp_single)./t_bpp*100; %Percentage compression of bi-polar plate

%%%%%%%%%%%%%%%%%% ASSEMBLY CLAMPING CONDITIONS %%%%%%%%%%%%%%%
% Three Different assembly clamping conditions are considered in this work
% 1: The displacement of the stack is fixed when operation conditions are
% applied, this is representative of infinitely stiff bolts.  All thermal
% expansion must be taken up without a change in the stack external length
%
%2: Free displacement of the stack.  The stack is free to expand as it
%transitions to the operating condition, this is representative of bolts
%with zero stiffness.
%
%3: Expansion under bolts of finite stiffness.  A combination of both
%methods 1 and 2, the stack can expand but additional force is exerted due
%to the stretching of the bolts

for m=1:N_cells %Find new pressure in each cell
    %Method 1: Fixed displacement  
    P_mea_change_fixed(m,:) = -P_mea(m,:)+interp1(compression_stack_total_thermal-delta_L_stack,P_mea_thermal(m,:),compression_stack_total); %Change in MEA clamping pressure due to thermal effects [N/m^2]  
    P_gasket_change_fixed(m,:) = -P_gasket(m,:)+interp1(compression_stack_total_thermal-delta_L_stack,P_gasket_thermal(m,:),compression_stack_total); %Change in gasket clamping pressure due to thermal effects [N/m^2]
    %Method 2: Free displacement
    compression_pem_free_thermal(m,:) = (compression_pem_single_thermal(m,:))/t_pem_array(m)*100; %Percentage compression of polymer electrolyte membrane
    compression_gdl_free_thermal(m,:) = (compression_gdl_single_thermal(m,:))/t_gdl_array(m)*100; %Percentage compression of gas diffusion layer
    compression_gasket_free_thermal(m,:) = (compression_gasket_single_thermal(m,:))/t_gasket_array(m)*100; %Percentage compression of gasket
end

%Method 1: Fixed
P_mea_new_fixed = P_mea+P_mea_change_fixed; %MEA clamping pressure after thermal effects (fixed) [N/m^2]
P_mea_change_fixed_perc = P_mea_change_fixed./P_mea*100; %Percentage change in MEA clamping pressure due to thermal effects (Fixed displacement) 
P_gasket_change_perc_fixed = P_gasket_change_fixed./P_gasket*100; %Percentage change in gasket clamping pressure due to thermal effects (fixed) [N/m^2]
P_gasket_new_fixed = P_gasket_change_fixed+P_gasket; %Gasking pressure change after thermal effects (fixed) [N/m^2]  
P_clamp_change_fixed = -P_clamp+interp1(compression_stack_total_thermal-delta_L_stack,P_clamp_thermal,compression_stack_total);
P_clamp_change_fixed_perc = P_clamp_change_fixed./P_clamp*100; %Percentage change in clamping pressure due to thermal effects (fixed)
P_clamp_new_fixed = P_clamp+P_clamp_change_fixed; %Stack clamping pressure after thermal effects (fixed) [N/m^2]
F_clamp_new_fixed = P_clamp_new_fixed*A_bpp_main; %Stack clamping force after thermal effects (fixed) [N]
%Method 2: Free (Constant force, find stack compression under operational conditions for equivalent load)
non_thermal_displacement_vector = linspace(min(compression_stack_total),max(min(compression_stack_total,compression_stack_total_thermal-delta_L_stack)),40); %Vector of original displacements [m]
F_compression = interp1(compression_stack_total,F_range_cell,non_thermal_displacement_vector);
compression_stack_free_thermal = interp1(F_range_cell_thermal,compression_stack_total_thermal-delta_L_stack,F_compression,'linear',0);

%Method 3: %Variable load variable displacement (Thermal loadcauses combination of expansion and load changes due to stiffness of clamping method)
k_clamp = E_clamp*A_clamp/t_stack; %Stiffness of clamping mechanism based on original stack stiffness [N/m]

%Find new displacement of stack after hygrothermal constraint applied for chosen original displacements,
%For known stack compression and force (non thermal), the additional clamping load for expanding
%The stack is found using k_clamp.  The point at which this intersects the
%thermal clamping curve is the new stack compression and clamping load
%Method only considers when the stack temperature is increased, not cooled
compression_stack_total_thermal_tied = zeros(size(non_thermal_displacement_vector)); %Produce zeros vector for loop variable
x_clamping = linspace(-delta_L_stack,max([compression_stack_total compression_stack_total_thermal]+delta_L_stack),length(compression_stack_total_thermal)); %'Stretching required by clamping mechanism to give desired clamping load (does not consider change in stiffness with elongation) [m]

for n=1:length(non_thermal_displacement_vector)
    non_thermal_displacement = non_thermal_displacement_vector(n); %Original stack compression before thermal constraints [m] 
    F_clamping = -k_clamp*x_clamping+interp1(compression_stack_total,F_range_cell,non_thermal_displacement)+k_clamp*non_thermal_displacement;
    F_thermal = interp1(compression_stack_total_thermal-delta_L_stack,F_range_cell_thermal,x_clamping,'linear',0);
    F_thermal(end) = F_clamping(end)+1; %Prevent no solution in extreme cases
    clamp_thermal_intersect = find(F_clamping<F_thermal,1);
    F_clamping_tied_2 = F_clamping(clamp_thermal_intersect);
    compression_stack_total_thermal_tied(n) = x_clamping(clamp_thermal_intersect); %New stack displacement [m] 
end

F_clamp_tied = interp1(compression_stack_total_thermal-delta_L_stack,F_range_cell_thermal,compression_stack_total_thermal_tied); %New stack clamping load [N] 
Bolt_stretching = non_thermal_displacement_vector-compression_stack_total_thermal_tied;
Bolt_stretchin_perc = Bolt_stretching/t_stack*100;

%Initialise loop variables
P_mea_original = zeros(N_cells,length(non_thermal_displacement_vector)); P_mea_thermal_bolt  = zeros(N_cells,length(non_thermal_displacement_vector)); P_gasket_original = zeros(N_cells,length(non_thermal_displacement_vector));
%Loop to get vaiables in displacement vs. pressure array
for m=1:N_cells
    P_mea_original(m,:) = interp1(compression_stack_total,P_mea(m,:),non_thermal_displacement_vector); %Original MEA clamping pressure at displacement array [N/m^2]
    P_mea_thermal_bolt(m,:) = interp1(compression_stack_total_thermal-delta_L_stack,P_mea_thermal(m,:),compression_stack_total_thermal_tied); %New clamping pressure after thermal expansion [N/m^2]
    P_gasket_original(m,:) = interp1(compression_stack_total,P_gasket(m,:),non_thermal_displacement_vector); %Original gasket clamping pressure at displacement array [N/m^2]
end

F_mea_thermal_bolt = P_mea_thermal_bolt*A_pem_cell; %Force on MEA after thermal effects [N]
F_gasket_thermal_bolt = repmat(F_clamp_tied,N_cells,1)-F_mea_thermal_bolt; %Force on gasket after thermal effects [N]

P_mea_change_bolt = P_mea_thermal_bolt-P_mea_original; %Change in MEA clamping pressure due to thermal effects [N/m^2]
P_mea_change_bolt_perc = P_mea_change_bolt./P_mea_original*100; %Percentage change in MEA clamping pressure due to thermal effects (bolted)   
P_gasket_thermal_bolt= F_gasket_thermal_bolt/A_gasket; %Gasket pressure after thermal effects [N/m^2]
P_gasket_change_bolt = P_gasket_thermal_bolt-P_gasket_original; %Change in gasket clamping pressure due to thermal effects [N/m^2]
F_clamp_original = interp1(compression_stack_total,F_range_cell,non_thermal_displacement_vector); %Original clamping force at displacement array [N]
F_clamp_change_bolt = F_clamp_tied-F_clamp_original; %Change in clamping force due to thermal effects [N]
F_clamp_change_bolt_perc = F_clamp_change_bolt./F_clamp_original*100; %Percentage change in clamping force due to thermal effects (bolted)
P_clamp_new_bolt = F_clamp_tied/A_bpp_main; %Clamping pressure after thermal effects [N/m^2]
P_clamp_change_bolt = F_clamp_change_bolt/A_bpp_main; %Change in clamping pressure due to thermal effects [N/m^2]

%%%%% Outputs %%%%%
mean_compression_gdl = mean(compression_gdl);
mean_compression_pem = mean(compression_pem);
mean_compression_gasket = mean(compression_gasket);
mean_P_mea = mean(P_mea);
mean_P_gasket = mean(P_gasket);

return %Remove return to produce non-thermal plots
%Stack clamping pressure vs compression
n_markers = 20;
n_points = 200;
max_P_clamp = 2e6;
max_P_clamp_length = find(P_clamp>max_P_clamp,1);
P_clamp_step = round(max_P_clamp_length/n_markers); %Steps per marker
P_clamp_step2 = round(max_P_clamp_length/n_points); %Steps per data point
figure,hold on,plot(P_clamp(1)/1e6,compression_stack(1),'ko-',P_clamp(1)/1e6,compression_bpp(1),'bx-',P_clamp(1)/1e6,mean_compression_gdl(1),'rd-',P_clamp(1)/1e6,mean_compression_pem(1),'gs-',P_clamp(1)/1e6,mean_compression_gasket(1),'kv-')
hold on,plot(P_clamp(1:P_clamp_step:max_P_clamp_length)/1e6,compression_stack(1:P_clamp_step:max_P_clamp_length),'ko',P_clamp(1:P_clamp_step:max_P_clamp_length)/1e6,compression_bpp(1:P_clamp_step:max_P_clamp_length),'bx',P_clamp(1:P_clamp_step:max_P_clamp_length)/1e6,mean_compression_gdl(1:P_clamp_step:max_P_clamp_length),'rd',P_clamp(1:P_clamp_step:max_P_clamp_length)/1e6,mean_compression_pem(1:P_clamp_step:max_P_clamp_length),'gs',P_clamp(1:P_clamp_step:max_P_clamp_length)/1e6,mean_compression_gasket(1:P_clamp_step:max_P_clamp_length),'kv')
hold on,plot(P_clamp(1:P_clamp_step2:max_P_clamp_length)/1e6,compression_stack(1:P_clamp_step2:max_P_clamp_length),'k-',P_clamp(1:P_clamp_step2:max_P_clamp_length)/1e6,compression_bpp(1:P_clamp_step2:max_P_clamp_length),'b-',P_clamp(1:P_clamp_step2:max_P_clamp_length)/1e6,mean_compression_gdl(1:P_clamp_step2:max_P_clamp_length),'r-',P_clamp(1:P_clamp_step2:max_P_clamp_length)/1e6,mean_compression_pem(1:P_clamp_step2:max_P_clamp_length),'g-',P_clamp(1:P_clamp_step2:max_P_clamp_length)/1e6,mean_compression_gasket(1:P_clamp_step2:max_P_clamp_length),'k-')
xlabel('Stack clamping pressure [MPa]'),ylabel('Compression [%]')
legend('Stack','BPP','GDL','PEM','Gasket','location','northwest'),box on
xlim([0 max_P_clamp/1e6])

%Stack clamping pressure vs. MEA/Gasket clamping pressure
figure,hold on,plot(P_clamp(1)/1e6,mean_P_mea(1)/1e6,'ko-',P_clamp(1)/1e6,mean_P_gasket(1)/1e6,'bx-')
hold on,plot(P_clamp(1:P_clamp_step:max_P_clamp_length)/1e6,mean_P_mea(1:P_clamp_step:max_P_clamp_length)/1e6,'ko',P_clamp(1:P_clamp_step:max_P_clamp_length)/1e6,mean_P_gasket(1:P_clamp_step:max_P_clamp_length)/1e6,'bx')
hold on,plot(P_clamp(1:P_clamp_step2:max_P_clamp_length)/1e6,mean_P_mea(1:P_clamp_step2:max_P_clamp_length)/1e6,'k-',P_clamp(1:P_clamp_step2:max_P_clamp_length)/1e6,mean_P_gasket(1:P_clamp_step2:max_P_clamp_length)/1e6,'b-')
xlabel('Stack clamping pressure [MPa]'),ylabel('Clamping Pressure [MPa]')
legend('MEA','Gasket','location','northwest'),box on
xlim([0 max_P_clamp/1e6])

